/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.addConnectionPoint;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.getConnectionPoints;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUIUtils.removeConnectionPoint;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewer;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.IController;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.LinkControllerBase;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.ILinkMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IMVCBundle;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;

/**
 * {@link IController} for {@link ILayoutedModelElement}s representing links and connections in a
 * {@link DiagramViewer}.
 * 
 * @author hoelzl
 */
public class LayoutedModelElementBasedLinkBendPointController<T extends ILayoutedModelElement>
		extends LinkControllerBase {
	/** {@link IModelChangeProvider} for this controller */
	private final LayoutModelElementModelChangeProvider layoutModelChangeProvider;

	/** Constructor. */
	public LayoutedModelElementBasedLinkBendPointController(ILinkMVCBundle mvcb,
			Class<T> modelType) {
		super(mvcb);

		Object model = mvcb.getModel();
		if(model == null) {
			throw new IllegalArgumentException("The given model is null!");
		}
		if(!modelType.isAssignableFrom(model.getClass())) {
			throw new IllegalArgumentException("Expected model of type " +
					modelType.getSimpleName() + ", but was " + model.getClass().getSimpleName());
		}

		ILayoutedModelElement lme = (ILayoutedModelElement)mvcb.getModel();
		this.layoutModelChangeProvider = new LayoutModelElementModelChangeProvider(lme);
	}

	/** Returns the model element. */
	@SuppressWarnings("unchecked")
	public T getModelElement() {
		return (T)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public IModelChangeProvider getModelChangeProvider() {
		return layoutModelChangeProvider;
	}

	/** {@inheritDoc} */
	@Override
	protected int getNumerOfBendPoints() {
		return getBendPointList(getModelElement()).size();
	}

	/** Returns the list of bend-points. */
	private EList<Point> getBendPointList(ILayoutedModelElement modelElement) {
		return getConnectionPoints(modelElement).getPoints();
	}

	/** {@inheritDoc} */
	@Override
	protected void createBendPointAt(int bpIndex, DiagramCoordinate location) {
		int x = (int)location.getX();
		int y = (int)location.getY();
		addConnectionPoint(getModelElement(), bpIndex, x, y);
	}

	/** {@inheritDoc} */
	@Override
	protected void updateModelAfterBendPointMove(int bpIndex, double dx, double dy) {
		List<Point> oldPoints = getBendPointList(getModelElement());
		Point point = oldPoints.get(bpIndex);
		int oldX = point.getX();
		int oldY = point.getY();
		int newX = oldX + (int)dx;
		int newY = oldY + (int)dy;
		point.setX(newX);
		point.setY(newY);
	}

	/** {@inheritDoc} */
	@Override
	protected void deleteBendPoint(int bpIndex) {
		removeConnectionPoint(getModelElement(), bpIndex);
	}

	/** {@inheritDoc} */
	@Override
	protected void updateModelAfterLinkMove(ILinkMVCBundle linkBundle, IMVCBundle startBundle,
			IMVCBundle endBundle) {
		// ignored
	}

	/** {@inheritDoc} */
	@Override
	protected void deleteLink() {
		IConnectionCompositorService.getInstance().disconnect(getModelElement());
	}

	/** {@inheritDoc} */
	@Override
	public void delete() {
		deleteLink();
	}
}
