/*-------------------------------------------------------------------------+
| Copyright 2019 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.visuals;

import static java.lang.Math.min;
import static org.fortiss.tooling.base.ui.editor.fx.visuals.CoordinateCorrections.RECTANGLE_INSETS;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.getNodeBounds;

import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.layout.IOffsetLayout;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.layout.ISideLayout;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.IContentAnchorageVisual;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.rectangular.RectangularBorderLocation;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.visual.rectangular.RectangularContentVisualBase;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.Rectangle;
import org.fortiss.tooling.kernel.model.INamedElement;

import javafx.geometry.Dimension2D;
import javafx.geometry.Rectangle2D;
import javafx.geometry.Side;

/**
 * {@link RectangularContentVisualBase} providing a default visual for boxes in standard
 * boxes-and-wires diagrams based on {@link ILayoutedModelElement}s.
 * 
 * @author munaro
 */
public abstract class LayoutedRectangularContentVisualBase<T extends ILayoutedModelElement & INamedElement>
		extends RectangularContentVisualBase {

	/** Constructor. */
	public LayoutedRectangularContentVisualBase(IContentMVCBundle mvcb, Class<T> modelType) {
		super(mvcb);

		// TODO(#3877): Move type checks to a common base class.
		Object model = mvcb.getModel();
		if(model == null) {
			throw new IllegalArgumentException("The given model is null!");
		}
		if(!modelType.isAssignableFrom(model.getClass())) {
			throw new IllegalArgumentException("Expected model of type " +
					modelType.getSimpleName() + ", but was " + model.getClass().getSimpleName());
		}
	}

	/** {@inheritDoc} */
	@Override
	public Rectangle2D getModelBounds() {
		Rectangle rectangle = getNodeBounds(getModelElement());
		double insetRadius = RECTANGLE_INSETS.getWidth();
		double insetDiameter = insetRadius * 2;
		return new Rectangle2D(rectangle.getX() + insetRadius, rectangle.getY() + insetRadius,
				rectangle.getWidth() - insetDiameter, rectangle.getHeight() - insetDiameter);
	}

	/** {@inheritDoc} */
	@Override
	public DiagramCoordinate getAnchorageLocation(IContentAnchorageVisual visual) {
		Rectangle2D contentRectangle = getCurrentBounds();
		Side rectangleSide = visual.getLayout(ISideLayout.class).getSide();
		double sideOffset = visual.getLayout(IOffsetLayout.class).getOffset();
		Dimension2D anchorageDimensions = visual.getDimensions();
		DiagramCoordinate location = new RectangularBorderLocation(rectangleSide, sideOffset,
				contentRectangle, anchorageDimensions).getLocation();
		double x = min(location.getX(),
				contentRectangle.getWidth() - anchorageDimensions.getWidth() / 2);
		double y = min(location.getY(),
				contentRectangle.getHeight() - anchorageDimensions.getHeight() / 2);
		return new DiagramCoordinate(x, y);
	}

	/** {@inheritDoc} */
	@Override
	protected double getBorderWidth() {
		return 1;
	}

	/** {@inheritDoc} */
	@Override
	protected double getOpacity() {
		return 1.0;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requireSelectionForMoveGesture() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requireSelectionForResizeGesture() {
		return false;
	}

	/** Return the {@link ILayoutedModelElement}. */
	// TODO(#3877): Move type checks to a common base class.
	@SuppressWarnings("unchecked")
	protected T getModelElement() {
		// Safe wild cast due to type check in constructor
		return (T)getModel();
	}

	/** {@inheritDoc} */
	@Override
	protected String getName() {
		return getModelElement().getName();
	}
}
